-- Debonucing filter 
-- Input active low   PB_L


LIBRARY ieee;     
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

			
ENTITY deboucing_filter IS
      Port (	CLK 	: IN	STD_LOGIC;
				PB_L		: IN	STD_LOGIC;
				CD   	: IN STD_LOGIC;
				QA 		: OUT	STD_LOGIC;
				QB		: OUT	STD_LOGIC
	);

END deboucing_filter;


ARCHITECTURE FSM_like OF deboucing_filter IS


	TYPE State_type IS (Idle, One_zero_detected, Two_zeroes_detected, Three_zeroes_detected, Key_presed, One_high);  

	ATTRIBUTE syn_encoding : STRING;
	ATTRIBUTE syn_encoding OF State_type : TYPE IS "sequential";

	SIGNAL present_state, future_state	: State_type ; 

BEGIN


state_register: PROCESS (CD, CLK)
	BEGIN
						
	IF CD = '1' THEN 			-- reset counter
		present_state <= idle; 
	ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous register (D-type flip-flop)
		present_state <= future_state; 
	END IF;

END PROCESS state_register;

------------------------- CC1: Combinational system for calculating next state

CC_1: PROCESS (present_state, PB_L)
	BEGIN
	   
		CASE present_state IS
				
			WHEN Idle =>
			   IF (PB_L = '1') THEN
			      	future_state <= Idle; 
				ELSE    
					future_state <= One_zero_detected;
				END IF;
				
			WHEN One_zero_detected => 
				IF (PB_L = '1') THEN
					future_state <= Idle; 
				ELSE    
					future_state <= Two_zeroes_detected ;
				END IF;
				
			WHEN Two_zeroes_detected => 
			   IF (PB_L = '1') THEN
			      future_state <= idle; 
				ELSE    
					future_state <= Three_zeroes_detected ;
				END IF;
				
			WHEN Three_zeroes_detected => 
					future_state <= key_presed ;
					
			WHEN Key_presed => 
			   IF (PB_L = '0') THEN
			      future_state <= Key_presed; 
				ELSE    
					future_state <= One_high ;
				END IF;
				
			WHEN One_high => 
			   IF (PB_L = '1') THEN
			      future_state <= Idle; 
				ELSE    
					future_state <= Key_presed ;
				END IF;
				
		END CASE ;		

END PROCESS CC_1;

------------------------- CS_2: combinational system for calculating extra outputs 
-------------------------- and outputing the present state (the actual count)

CC_2: PROCESS (present_state)
	BEGIN

		CASE present_state IS
				WHEN Idle => 
					QA <= '0';
					QB <= '0';
				WHEN One_zero_detected => 
					QA <= '0';
					QB <= '0';
				WHEN Two_zeroes_detected => 
					QA <= '0';
					QB <= '0';
				WHEN Three_zeroes_detected => 
					QA <= '1';
					QB <= '1';
				WHEN Key_presed => 
					QA <= '0';
					QB <= '1';
				WHEN One_high => 
					QA <= '0';
					QB <= '1';
		END CASE ;		

END PROCESS CC_2;

-- Place other logic if necessary

END FSM_like;